package org.apidesign.apifest08.test;

import java.util.Currency;
import junit.framework.TestCase;
import org.apidesign.apifest08.currency.ConversionRate;
import org.apidesign.apifest08.currency.Convertor;
import org.apidesign.apifest08.currency.MonetaryAmount;
import org.apidesign.apifest08.currency.TableConvertor;

/** Finish the Convertor API, and then write bodies of methods inside
 * of this class to match the given tasks. To fullfil your task, use the
 * API define in the <code>org.apidesign.apifest08.currency</code> package.
 * Do not you reflection, or other hacks as your code
 * shall run without any runtime permissions.
 */
public class Task1Test extends TestCase {
    public Task1Test(String testName) {
        super(testName);
    }

    @Override
    protected void setUp() throws Exception {
    }

    @Override
    protected void tearDown() throws Exception {
    }

    protected static final Currency CZK = Currency.getInstance( "CZK" );
    protected static final Currency SKK = Currency.getInstance( "SKK" );
    protected static final Currency USD = Currency.getInstance( "USD" );
    
    /** Create convertor that understands two currencies, CZK and
     *  USD. Make 1 USD == 17 CZK.
     *
     * Creation of the convertor shall not require subclassing of any class
     * or interface on the client side.
     *
     * @return prepared convertor ready for converting USD to CZK and CZK to USD
     */
    public static Convertor createCZKtoUSD() {
        final TableConvertor convertor = new TableConvertor();
        final MonetaryAmount amountInCZK = new MonetaryAmount( 17, CZK );
        final MonetaryAmount amountInUSD = new MonetaryAmount( 1, USD );
        convertor.putIntoTable( new ConversionRate( amountInCZK, amountInUSD ) );
        convertor.putIntoTable( new ConversionRate( amountInUSD, amountInCZK ) );
        return new ContractImposingDelegatingConvertor( convertor ).test();
    }

    /** Create convertor that understands two currencies, CZK and
     *  SKK. Make 100 SKK == 80 CZK.
     *
     * Creation of the convertor shall not require subclassing of any class
     * or interface on the client side.
     * 
     * @return prepared convertor ready for converting SKK to CZK and CZK to SKK
     */
    public static Convertor createSKKtoCZK() {
        final TableConvertor convertor = new TableConvertor();
        final MonetaryAmount amountInSKK = new MonetaryAmount( 100, SKK );
        final MonetaryAmount amountInCZK = new MonetaryAmount( 80, CZK );
        convertor.putIntoTable( new ConversionRate( amountInSKK, amountInCZK ) );
        convertor.putIntoTable( new ConversionRate( amountInCZK, amountInSKK ) );
        return new ContractImposingDelegatingConvertor( convertor ).test();
    }

    /** Use the convertor from <code>createCZKtoUSD</code> method and do few conversions
     * with it.
     */
    public void testCurrencyCZKUSD() throws Exception {
        final Convertor c = createCZKtoUSD();
        
        // convert $5 to CZK using c:
        final Convertor.ConversionResult r1 = c.convert( new Convertor.ConversionRequest( new MonetaryAmount( 5, USD ), CZK ) );
        final MonetaryAmount a1 = r1.getNetAmount();
        // assertEquals("Result is 85 CZK");
        assertNotNull( a1 );
        assertEquals( 85.0, a1.getAmount().doubleValue() );
        assertEquals( CZK, a1.getCurrency() );

        // convert $8 to CZK
        final Convertor.ConversionResult r2 = c.convert( new Convertor.ConversionRequest( new MonetaryAmount( 8, USD ), CZK ) );
        final MonetaryAmount a2 = r2.getNetAmount();
        // assertEquals("Result is 136 CZK");
        assertNotNull( a2 );
        assertEquals( 136.0, a2.getAmount().doubleValue() );
        assertEquals( CZK, a2.getCurrency() );

        // convert 1003CZK to USD
        final Convertor.ConversionResult r3 = c.convert( new Convertor.ConversionRequest( new MonetaryAmount( 1003, CZK ), USD ) );
        final MonetaryAmount a3 = r3.getNetAmount();
        // assertEquals("Result is 59 USD");
        assertNotNull( a3 );
        assertEquals( 59.0, a3.getAmount().doubleValue() );
        assertEquals( USD, a3.getCurrency() );
    }

    /** Use the convertor from <code>createSKKtoCZK</code> method and do few conversions
     * with it.
     */
    public void testCurrencySKKCZK() throws Exception {
        final Convertor c = createSKKtoCZK();
        
        // convert 16CZK using c:
        final Convertor.ConversionResult r1 = c.convert( new Convertor.ConversionRequest( new MonetaryAmount( 16, CZK ), SKK ) );
        final MonetaryAmount a1 = r1.getNetAmount();
        // assertEquals("Result is 20 SKK");
        assertNotNull( a1 );
        assertEquals( 20.0, a1.getAmount().doubleValue() );
        assertEquals( SKK, a1.getCurrency() );
        
        // convert 500SKK to CZK
        final Convertor.ConversionResult r2 = c.convert( new Convertor.ConversionRequest( new MonetaryAmount( 500, SKK ), CZK ) );
        final MonetaryAmount a2 = r2.getNetAmount();
        // assertEquals("Result is 400 CZK");
        assertNotNull( a2 );
        assertEquals( 400.0, a2.getAmount().doubleValue() );
        assertEquals( CZK, a2.getCurrency() );
    }

}

