package org.apidesign.apifest08.currency;

import java.io.Serializable;

/**
 * Value class, holding an amount of the currency & an identifier of the currency.
 * Designed to be an immutable.
 * 
 * Because of a vague definition of types of the both fields,
 * the class has generic types, used as types of the fields.
 * These types should be immutable classes, too.
 * 
 * @author ked
 */
public final class CurrencyValue<AmountType, IdentifierType> implements Serializable {

    private final AmountType amount;
    private final IdentifierType identifier;

    private CurrencyValue(AmountType amount, IdentifierType identifier) {
        this.amount = amount;
        this.identifier = identifier;
    }

    public AmountType getAmount() {
        return amount;
    }

    public IdentifierType getIdentifier() {
        return identifier;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final CurrencyValue other = (CurrencyValue) obj;
        if (this.amount != other.amount && (this.amount == null || !this.amount.equals(other.amount))) {
            return false;
        }
        if (this.identifier != other.identifier && (this.identifier == null || !this.identifier.equals(other.identifier))) {
            return false;
        }
        return true;
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 97 * hash + (this.amount != null ? this.amount.hashCode() : 0);
        hash = 97 * hash + (this.identifier != null ? this.identifier.hashCode() : 0);
        return hash;
    }

    /**
     * Creates new instance.
     * Generic types of the new instance are derived from types of the parameters.
     * 
     * @param <AmountType> type of the currency amount
     * @param <IdentifierType> type of the currency identifier
     * @param amount currency amount
     * @param identifier currency identifier
     * @return new instance
     */
    public static <AmountType, IdentifierType> CurrencyValue<AmountType, IdentifierType> getCurrencyValue(AmountType amount, IdentifierType identifier) {
        return new CurrencyValue<AmountType, IdentifierType>(amount, identifier);
    }
}
