package org.apidesign.apifest08.test;

import junit.framework.TestCase;
import org.apidesign.apifest08.currency.Convertor;
import org.apidesign.apifest08.currency.CurrencyValue;

/** Finish the Convertor API, and then write bodies of methods inside
 * of this class to match the given tasks. To fullfil your task, use the
 * API define in the <code>org.apidesign.apifest08.currency</code> package.
 * Do not you reflection, or other hacks as your code
 * shall run without any runtime permissions.
 */
public class Task1Test extends TestCase {
    public Task1Test(String testName) {
        super(testName);
    }

    @Override
    protected void setUp() throws Exception {
    }

    @Override
    protected void tearDown() throws Exception {
    }

    /** Create convertor that understands two currencies, CZK and
     *  USD. Make 1 USD == 17 CZK.
     *
     * Creation of the convertor shall not require subclassing of any class
     * or interface on the client side.
     *
     * @return prepared convertor ready for converting USD to CZK and CZK to USD
     */
    public static Convertor<Integer, String> createCZKtoUSD() {
        return Convertor.getConvertorIntegerString(
                CurrencyValue.getCurrencyValue(1, "USD"),
                CurrencyValue.getCurrencyValue(17, "CZK")
        );
    }

    /** Create convertor that understands two currencies, CZK and
     *  SKK. Make 100 SKK == 80 CZK.
     *
     * Creation of the convertor shall not require subclassing of any class
     * or interface on the client side.
     * 
     * @return prepared convertor ready for converting SKK to CZK and CZK to SKK
     */
    public static Convertor<Integer, String> createSKKtoCZK() {
        return Convertor.getConvertorIntegerString(
                CurrencyValue.getCurrencyValue(100, "SKK"),
                CurrencyValue.getCurrencyValue(80, "CZK")
        );
    }
    
    /** Use the convertor from <code>createCZKtoUSD</code> method and do few conversions
     * with it.
     */
    public void testCurrencyCZKUSD() throws Exception {
        Convertor<Integer, String> c = createCZKtoUSD();
        
        CurrencyValue<Integer, String> result;
        
        // convert $5 to CZK using c:
        // assertEquals("Result is 85 CZK");
        result = c.convert(CurrencyValue.getCurrencyValue(5, "USD"));
        assertEquals(CurrencyValue.getCurrencyValue(85, "CZK"), result);

        // convert $8 to CZK
        // assertEquals("Result is 136 CZK");
        result = c.convert(CurrencyValue.getCurrencyValue(8, "USD"));
        assertEquals(CurrencyValue.getCurrencyValue(136, "CZK"), result);

        // convert 1003CZK to USD
        // assertEquals("Result is 59 USD");
        result = c.convert(CurrencyValue.getCurrencyValue(1003, "CZK"));
        assertEquals(CurrencyValue.getCurrencyValue(59, "USD"), result);
    }

    /** Use the convertor from <code>createSKKtoCZK</code> method and do few conversions
     * with it.
     */
    public void testCurrencySKKCZK() throws Exception {
        Convertor<Integer, String> c = createSKKtoCZK();
        
        CurrencyValue<Integer, String> result;
        
        // convert 16CZK using c:
        // assertEquals("Result is 20 SKK");
        result = c.convert(CurrencyValue.getCurrencyValue(16, "CZK"));
        assertEquals(CurrencyValue.getCurrencyValue(20, "SKK"), result);

        // convert 500SKK to CZK
        // assertEquals("Result is 400 CZK");
        result = c.convert(CurrencyValue.getCurrencyValue(500, "SKK"));
        assertEquals(CurrencyValue.getCurrencyValue(400, "CZK"), result);
    }
}

